/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class fluidsynth_Sample */

#ifndef _Included_fluidsynth_Sample
#define _Included_fluidsynth_Sample
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     fluidsynth_Sample
 * Method:    newSample
 * Signature: (Ljava/lang/String;I)I
 */
JNIEXPORT jint JNICALL Java_fluidsynth_Sample_newSample
  (JNIEnv *, jobject, jstring, jint);

/*
 * Class:     fluidsynth_Sample
 * Method:    deleteSample
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_fluidsynth_Sample_deleteSample
  (JNIEnv *, jobject, jint);

#ifdef __cplusplus
}
#endif
#endif
