/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class fluidsynth_Synth */

#ifndef _Included_fluidsynth_Synth
#define _Included_fluidsynth_Synth
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     fluidsynth_Synth
 * Method:    newSynth
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_fluidsynth_Synth_newSynth
  (JNIEnv *, jobject);

/*
 * Class:     fluidsynth_Synth
 * Method:    deleteSynth
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_fluidsynth_Synth_deleteSynth
  (JNIEnv *, jobject, jint);

/*
 * Class:     fluidsynth_Synth
 * Method:    add
 * Signature: (IIIIII)I
 */
JNIEXPORT jint JNICALL Java_fluidsynth_Synth_add
  (JNIEnv *, jobject, jint, jint, jint, jint, jint, jint);

/*
 * Class:     fluidsynth_Synth
 * Method:    remove
 * Signature: (IIII)I
 */
JNIEXPORT jint JNICALL Java_fluidsynth_Synth_remove
  (JNIEnv *, jobject, jint, jint, jint, jint);

JNIEXPORT jint JNICALL Java_fluidsynth_Synth_loadSoundFont
(JNIEnv *env, jobject obj, jint synth, jstring filename);

#ifdef __cplusplus
}
#endif
#endif
